from tkmfly.core import fly_load4, fly_root, fly_local, fly_chdir
from tkinter import Widget, PhotoImage, ttk


def load_history():
    from tkmfly.history.pkgIndex import code
    fly_root().eval(code)


def history_init(widget: Widget, len: int = 30):
    load_history()
    fly_root().call("::history::init", widget, len)


def history_add(widget: Widget, line):
    load_history()
    fly_root().call("::history::add", widget, line)


def history_get(widget: Widget):
    load_history()
    return fly_root().call("::history::get", widget)


def history_up(widget: Widget):
    load_history()
    return fly_root().call("::history::up", widget)


def history_down(widget: Widget):
    load_history()
    return fly_root().call("::history::down", widget)


def history_clear(widget: Widget):
    load_history()
    return fly_root().call("::history::clear", widget)


def history_remove(widget: Widget):
    load_history()
    return fly_root().call("::history::remove", widget)


def history_alery(widget: Widget):
    load_history()
    return fly_root().call("::history::alert", widget)


class History(object):
    def __init__(self, widget: Widget, len: int = 30):
        """
namespace eval history {

    bind History <Up>   {::history::up %W}

    bind History <Down> {::history::down %W}

}
        """

        self._w = widget
        self.init(widget, len)

    def init(self, widget: Widget, len: int = 30):
        """
proc ::history::init {w {len 30}} {

    variable history

    variable prefs

    set bt [bindtags $w]

    if {[lsearch $bt History] > -1} { error "$w already has a history" }

    if {[set i [lsearch $bt $w]] < 0} { error "cant find $w in bindtags" }

    bindtags $w [linsert $bt [expr {$i + 1}] History]

    array set history [list $w,list {} $w,cur -1]

    set prefs(maxlen,$w) $len

    return $w

}

        :param widget: target widget
        :param len: list len
        :return:
        """
        history_init(self._w, len)

    def remove(self):
        """
proc ::history::remove {w} {

    variable history

    variable prefs

    set bt [bindtags $w]

    if {[set i [lsearch $bt History]] < 0} { error "$w has no history" }

    bindtags $w [lreplace $bt $i $i]

    unset prefs(maxlen,$w) history($w,list) history($w,cur)

}
        :return:
        """

        history_remove(self._w)

    def up(self):
        """
proc ::history::up {w} {

    variable history

    if {[lindex $history($w,list) [expr {$history($w,cur) + 1}]] != ""} {

        if {$history($w,cur) == -1} {

            set history($w,tmp) [$w get]

        }

        $w delete 0 end

        incr history($w,cur)

        $w insert end [lindex $history($w,list) $history($w,cur)]

    } else {

        alert $w

    }

}

        :return:
        """
        history_up(self._w)

    def down(self):
        """
proc ::history::down {w} {

    variable history

    if {$history($w,cur) != -1} {

        $w delete 0 end

        if {$history($w,cur) == 0} {

            $w insert end $history($w,tmp)

            set history($w,cur) -1

        } else {

            incr history($w,cur) -1

            $w insert end [lindex $history($w,list) $history($w,cur)]

        }

    } else {

        alert $w

    }

}

        :return: None
        """

        history_down(self._w)

    def add(self, line: str):
        """
proc ::history::add {w line} {

    variable history

    variable prefs

    if {$history($w,cur) > -1 && [lindex $history($w,list) $history($w,cur)] == $line} {

        set history($w,list) [lreplace $history($w,list) $history($w,cur) $history($w,cur)]

    }

    set history($w,list) [linsert $history($w,list) 0 $line]

    set history($w,list) [lrange $history($w,list) 0 $prefs(maxlen,$w)]

    set history($w,cur) -1

}

        :param line: a line strings
        :return:
        """

        history_add(self._w, line)

    def get(self):
        """
proc ::history::add {w line} {

    variable history

    variable prefs

    if {$history($w,cur) > -1 && [lindex $history($w,list) $history($w,cur)] == $line} {

        set history($w,list) [lreplace $history($w,list) $history($w,cur) $history($w,cur)]

    }

    set history($w,list) [linsert $history($w,list) 0 $line]

    set history($w,list) [lrange $history($w,list) 0 $prefs(maxlen,$w)]

    set history($w,cur) -1

}
        :return:
        """
        return history_get(self._w)

    def clear(self):
        """
proc ::history::clear {w} {

    variable history

    set history($w,cur) -1

    set history($w,list) {}

    unset -nocomplain history($w,tmp)

}
        """
        history_clear(self._w)

    def alert(self):

        """
proc ::history::alert {w} {bell}
        """

        history_alery(self._w)


if __name__ == '__main__':
    from tkinter import Tk, Entry, ttk

    root = Tk()

    entry = ttk.Entry()
    entry.pack()

    history = History(entry)
    history.add(114514)
    history.add(3.1415926)
    history.up()

    print(history.get())
    print(type(history.get()))

    root.mainloop()



from tkmfly.core import fly_load4, fly_root, fly_local, fly_chdir
from tkinter import Widget, PhotoImage


def load_tooltip():
    from tkmfly.tooltip.pkgIndex import code
    fly_root().eval(code)


class Tooltip(Widget):
    def __init__(self, master=None):
        load_tooltip()

        super().__init__(master, "tooltip::tooltip")

    def tooltip(self, widget, tooltip: str = "tooltip"):
        fly_root().call("tooltip::tooltip", widget,
                        "--", tooltip)

    def fade(self, boolean: bool = True):
        self.tk.call("tooltip::tooltip", "fade", boolean)

    def delay(self, millisecs: int = 50):
        self.tk.call("tooltip::tooltip", "delay", millisecs)

    def clear(self, pattern: str = "defaults"):
        self.tk.call("tooltip::tooltip", "clear", "-pattern", pattern)


def tooltip_example_tcl():
    load_tooltip()
    fly_root().eval("""
# Demonstrate widget tooltip
package require tooltip
pack [label .l -text "label"]
tooltip::tooltip .l "This is a label widget"

# Demonstrate menu tooltip
package require tooltip
. configure -menu [menu .menu]
.menu add cascade -label Test -menu [menu .menu.test -tearoff 0]
.menu.test add command -label Tooltip
tooltip::tooltip .menu.test -index 0 "This is a menu tooltip"

# Demonstrate canvas item tooltip
package require tooltip
pack [canvas .c]
set item [.c create rectangle 10 10 80 80 -fill red]
tooltip::tooltip .c -item $item "Canvas item tooltip"

# Demonstrate listbox item tooltip
package require tooltip
pack [listbox .lb]
.lb insert 0 "item one"
tooltip::tooltip .lb -item 0 "Listbox item tooltip"

# Demonstrate text tag tooltip
package require tooltip
pack [text .txt]
.txt tag configure TIP-1 -underline 1
tooltip::tooltip .txt -tag TIP-1 "tooltip one text"
.txt insert end "An example of a " {} "tooltip" TIP-1 " tag.\n" {}
""")


if __name__ == '__main__':
    from tkinter import Tk, Entry, ttk

    root = Tk()

    tooltip_example_tcl()

    root.mainloop()


from tkinter import Widget


def _command(*args, **kwargs):
    pass


class Tree(Widget):
    """
    简易树视图控件

    根据https://wiki.tcl-lang.org/page/A+Minimal+Tree+Widget文章修改而来
    """

    def __init__(self, master=None):
        """
        简易示例：

        from tkinter import *
        from tkdev5 import *


        root = Tk()

        tree = DevMinimalTreeView()
        tree.add(0, "First", lambda: print("First"))
        tree.add(1, "Second", lambda: print("Second"))
        tree.add(2, "Third", lambda: print("Third"))
        tree.toggle(1)
        tree.toggle(2)

        root.mainloop()
        """

        MinimalTree = """
 namespace eval ::ttree {}
 image create photo ::ttree::tree-close -data {
   R0lGODdhCQAJAJEAAGZmZv///wAAAP///ywAAAAACQAJAAACGYSPFD8A8IDA
   +AEQJDsAKH4A4AGB8YPgYwEAOw== 
 }
 image create photo ::ttree::tree-open -data {
   R0lGODdhCQAJAJEAAGZmZv///wAAAP///ywAAAAACQAJAAACHoSPFD8ATo6A
   REhEEBCSHQARQUoCICJISQBQ/CD4WAA7 
 }
 image create photo ::ttree::tree-dot -data {
   R0lGODdhCQAJAJEAAP///wAAAP///////ywAAAAACQAJAAACD4SPqXtB8CEg
   8CiEj6l7BQA7 
 }

 proc ::ttree::tree {w args} {
   variable tree;

   ::text $w -cursor {} -spacing3 1p
   if { [llength $args] > 0 } {
        $w configure {*}$args
      }
   set tree($w,items) 0
   set tags [bindtags $w]
   set pos [lsearch -exact $tags "Text"]
   bindtags $w [lreplace $tags $pos $pos]
   bind $w <Destroy> [list array unset ::ttree::tree %W,*]

   $w tag configure sub-0 -elide 0

   return $w;
 };# ttree::tree

 proc ::ttree::add {w parent txt cmd} {
   variable tree;

   if { ![winfo exists $w] || ![info exists tree($w,items)] } {
        error "widget \"$w\" does not exist, or is not a tree widget"
      }

   if { $parent != 0 && ![info exists tree($w,parent,$parent)] } {
        error "tree \"$w\" has no id \"$parent \""
      }

   return [addSub $w $parent $txt $cmd];
 }

 proc ::ttree::addSub {w parent txt cmd} {
   variable tree;

   set new [incr tree($w,items)]
   set tree($w,parent,$new) $parent
   set taglist [list id-$new]
   set tagParent $parent
   while { [info exists tree($w,parent,$tagParent)] } {
           lappend taglist sub-$tagParent
           set tagParent $tree($w,parent,$tagParent)
         }
   $w tag configure id-$new -lmargin1 "[expr {13 * ([llength $taglist]-1) }]p"
   if { $parent != 0 && ![info exists tree($w,children,$parent)] } {
        setUpParent $w $parent
      }
   lappend tree($w,children,$parent) $new
   if { $parent == 0 } {
        set where end
      } else {
        if { [catch {$w index sub-$parent.last} where] } {
             set where [$w index id-$parent.last]
           }
      }
   $w insert $where " $txt\n" $taglist
   set where [$w index id-$new.first+1char]
   $w image create $where -image ::ttree::tree-dot -align center -pady 2 -padx 4
   $w tag add btn-$new $where
   if { $cmd != "" } {
        $w tag bind id-$new <1> $cmd
      }       
   return $new;
 };# ttree::addSub

 proc ::ttree::setUpParent {w parent} {
   variable tree;

   $w tag configure sub-$parent -elide 1 
   set tree($w,elide,$parent) 1
   $w tag lower btn-$parent
   $w tag lower sub-$parent 
   $w tag bind btn-$parent <Button-1> [list ::ttree::toggle $w $parent]
   $w image configure btn-$parent.first -image ::ttree::tree-open

 };# ttree::setUpParent

 proc ::ttree::toggle {w parent} {
   variable tree;

   set base [expr {!$tree($w,elide,$parent)}]
   set tree($w,elide,$parent) $base
   $w image configure btn-$parent.first -image [expr {$base ? "::ttree::tree-open" : "::ttree::tree-close"}]
   $w tag configure sub-$parent -elide [expr { $base ? 1 : "" }]

 };# ttree::toggle

 proc ::ttree::show {w id} {
   variable tree;

   while { [info exists tree($w,parent,$id)] } {
      set id $tree($w,parent,$id)
      if { $id == 0 } {
           break;
         }
      if { $tree($w,elide,$id) } {
           toggle $w $id
         }
   }
 };# ttree::show

 # Optional, export [tree] and [add] into the global namespace
 namespace eval ::ttree {namespace export tree add}
 namespace import ::ttree::*

        """
        from tkinter import _default_root
        _default_root.eval(MinimalTree)
        if master is None:
            master = _default_root
        super().__init__(master, "::ttree::tree")

    def add(self, parent: int = 0, text: str = "", command=_command):
        """
        添加列

        Attributes:
            parent(Widget): 列所在行
            text(int): 列文本
            command: 点击事件
        """
        from tkinter import _default_root
        command = _default_root.register(command)
        _default_root.call("::ttree::add", self, parent, text, command)

    def toggle(self, parent: Widget = 0):
        """
        切换。将收起的列展开

        Attributes:
            parent(Widget): 展开行
        """
        from tkinter import _default_root
        _default_root.call("::ttree::toggle", self, parent)

    def setup_parent(self, parent: Widget = 0):
        from tkinter import _default_root
        _default_root.call("::ttree::setUpParent", self, parent)

    def show(self, id: Widget = 0):
        from tkinter import _default_root
        _default_root.call("::ttree::show", self, id)

    def testtk(self):
        from tkinter import _default_root
        _default_root.eval("""
pack [ttree::tree .t]
ttree::add .t 0 Foo {puts "foo"}
ttree::add .t 0 Bar {puts "bar"}
ttree::add .t 1 Baz {}
ttree::add .t 1 Bleep {puts "bleep"}
ttree::add .t 0 Bloop {puts "bloop"}
ttree::add .t 0 Splash {puts "splash"}
ttree::add .t 3 Boing {puts "boing"}
ttree::add .t 3 Sprocket {puts "sprocket"}
ttree::add .t 3 Meep {puts "meep"}
""")

    def test(self):
        t = Tree()
        t.add(0, "Foo", lambda: print("foo"))
        t.add(0, "Bar", lambda: print("bar"))
        t.add(1, "Baz")
        t.add(1, "Bleep", lambda: print("bleep"))
        t.add(0, "Bloop", lambda: print("bloop"))
        t.add(0, "Splash", lambda: print("splash"))
        t.add(3, "Boing", lambda: print("boing"))
        t.add(3, "Sprocket", lambda: print("sprocked"))
        t.add(3, "Meep", lambda: print("meep"))
        t.pack()


if __name__ == '__main__':
    import tkinter as tk
    root = tk.Tk()
    ttree = Tree()
    ttree.test()
    root.mainloop()


# From https://github.com/m0x3/Python3-Mutliple-Document-Interface/tree/master

try:
    import Pmw
except:
    pass
else:
    from tkmfly.tkmdi.Widgets.MDI import MDIChild as FlyXMDIChild, MDIParent as FlyXMDIParent
    from tkmfly.tkmdi.Widgets.iconPath import path as FlyXMDIIconPath
    from tkmfly.tkmdi.Widgets.FlatButtons import Flatbutton as FlyXFlatButton, FlatRadiobutton as FlyXFlatRadioButton, \
        FlatRadiogroup as FlyXFlatRadioGroup
    from tkmfly.tkmdi.Widgets.Toolbar import Toolbar as FlyXToolBar
    from tkmfly.tkmdi.Widgets.Tree import Tree as FlyXTreeView
    from tkmfly.tkmdi.Widgets.ProgressBar import ProgressBar as FlyXProgressBar


from tkinter import *
import time

class ProgressBar(Label):
    def __init__(self, parent, totalticks=1, spacing=4, color="#00008b", 
                 relief='sunken', bd=1, width=120, height=12, bg='grey',
                 textcolor='white', font=("MS Sans Serif", 8), **kw):
        Label.__init__(self, parent, kw, relief=relief, bd=bd, bg=bg)
        self.pack(side='left', anchor=NW, padx=2)       
        self.parent = parent
        self.canvas = Canvas(self, width=width, height=height)        
        self.canvas.pack(side='left', anchor=NW)
        self.totalticks, self.spacing = totalticks, spacing
        self.percentage = self.ticks = 0         
        self.color, self.bg = color, bg
        self.textcolor, self.font = textcolor, font

    def checkIfDone(self):
        self.canvas.update_idletasks()
        if self.percentage >= 0.99 or self.ticks >= self.totalticks:
            time.sleep(0.05)
            self.canvas.delete(ALL)
            self.canvas.update_idletasks()

    def doPercentage(self, percentage=None, showText='true'):
        self.canvas.delete(ALL)
        w, h = self.winfo_width(), self.winfo_height()
        self.canvas.create_rectangle(0, 0, w*percentage, h,
            fill=self.color, outline=self.color)
        if showText == 'true':
            msg = str(int(percentage*100)) + "%"
            self.canvas.create_text(w/2+1, h/2, text=msg, fill='black', 
                font=self.font)
            self.canvas.create_text(w/2, h/2-1, text=msg, fill=self.textcolor, 
                font=self.font)
        self.checkIfDone()

    def doTicks(self, ticks=None):
        self.canvas.delete(ALL)
        sp = self.spacing
        w = self.winfo_width()/self.totalticks - sp
        h = self.winfo_height()
        self.ticks = ticks
        for i in range(0,self.ticks):
            self.canvas.create_rectangle(i*(w+sp), 0, (i+1)*w + i*sp, h, 
                fill=self.color, outline=self.color)      
        self.checkIfDone()

    def addTick(self):
        self.canvas.delete(ALL)          
        sp = self.spacing
        w = self.winfo_width()/self.totalticks - sp
        h = self.winfo_height()
        self.ticks = self.ticks + 1
        for i in range(0,self.ticks):
            self.canvas.create_rectangle(i*(w+sp), 0, (i+1)*w + i*sp, h, 
                fill=self.color, outline=self.color) 
        self.checkIfDone()


def testpercentage(increment=0.01, totaltime=2):
    steps = totaltime/increment
    for i in range(steps):
        bar.doPercentage(i/steps)
        time.sleep(increment)

def testticks(increment=0.3, totalticks=10):
    bar.totalticks = totalticks
    for i in range(totalticks):
        bar.doTicks(i)
        time.sleep(increment)

def testaddtick(increment=0.3, totalticks=15):
    bar.totalticks = totalticks
    bar.ticks = 0
    for i in range(totalticks):
        bar.addTick()
        time.sleep(increment)

def tests():
    testpercentage()
    time.sleep(0.1)
    testticks()
    time.sleep(0.1)
    testaddtick()

if __name__ == '__main__':
    root = Tk() 
    root.title('ProgressBar test')
    bar = ProgressBar(root)
    bar.after(1, tests)
    root.mainloop()


from tkmfly.core import *

from tkmfly.notifywindow import NotifyWindow as FlyNotifyWindow
from tkmfly.tooltip import Tooltip as FlyToolTip
from tkmfly.datefield import DateField as FlyDateField
from tkmfly.history import History as FlyHistory
from tkmfly.ttree import Tree as FlySimpleTree

from tkmfly.tkmdi import *


import contextlib
import os

__all__ = [
    "fly_chdir",
    "fly_local",
    "fly_root",
    "fly_load1",
    "fly_load2",
    "fly_load3",
    "fly_load4",
    "fly_load5"
]


def _chdir(target: str):
    @contextlib.contextmanager
    def chdir(taget: str):
        current = os.getcwd()
        try:
            os.chdir(target.replace("/", "\\"))
            yield
        finally:
            os.chdir(current)

    return chdir(target)


fly_chdir = _chdir


def _local():
    return os.path.abspath(os.path.dirname(__file__))


fly_local = _local

from tkinter import Tk


def _root() -> Tk:
    from tkinter import _default_root
    return _default_root


fly_root = _root


def _load1(package: str):
    """
    导入指定库

    :param package: 包名
    :return:
    """

    fly_root().eval(f"package require {package}")


fly_load1 = _load1


def _load2(pkg_index: str = "pkgIndex.tcl"):
    """
    加载库

    :param pkg_index: 包索引
    :return:
    """
    fly_root().eval("set dir [file dirname [info script]]")
    fly_root().eval(f"source {pkg_index}")


fly_load2 = _load2


def _load3(package: str, pkg_index: str = "pkgIndex.tcl"):
    """
    加载并导入库

    :param package: 包名
    :param pkg_index: 包索引
    :return:
    """
    _load2(pkg_index)
    _load1(package)


fly_load3 = _load3


def _load4(package: str, local, pkg_index: str = "pkgIndex.tcl"):
    """
    加载指定位置下的包

    :param package: 包名
    :param local: 包位置
    :param pkg_index: 包索引
    :return:
    """

    with fly_chdir(local):
        _load3(package, pkg_index)


fly_load4 = _load4


def _load5(local, pkg_index: str = "pkgIndex.tcl"):
    """
    仅加载不导入

    :param local: 包位置
    :param pkg_index: 包索引
    :return:
    """
    with fly_chdir(local):
        _load2(pkg_index)


fly_load5 = _load5


def _load6(package: str, local):
    """
    仅加载不导入

    :param local: 包位置
    :param package: 包名
    :return:
    """
    with fly_chdir(local):
        _load1(package)


fly_load6 = _load6



from tkinter import *
import time
from tkmfly.tkmdi.Widgets.iconPath import path

class Flatbutton(Label):
    def __init__(self, parent, imagefile=None, command=None, statushelp='', balloonhelp='',
                 statusbar=None, balloon=None, orient='horz', ht=21, wd=21, bd=1, 
                 activebackground='lightgrey', padx=0, pady=0, state='normal'):
        Label.__init__(self, parent, height=ht, width=wd, relief='flat', bd=bd)
        self.bg = self.cget('bg')
        self.activebackground = activebackground
        if imagefile != None:
            self.Icon = PhotoImage(file=imagefile)
            self.config(image=self.Icon)
        self.command = command
        self.bind("<Enter>", self.enter)
        self.bind("<Leave>", self.leave)
        self.bind("<ButtonPress-1>", self.b1down)
        self.bind("<ButtonRelease-1>", self.b1up)
        if statusbar != None:
            statusbar.bind(self, statushelp)
        if balloon != None:
            balloon.bind(self, balloonhelp)
        if orient == 'horz':
            self.pack(side='left', anchor=NW, padx=padx, pady=pady)
        elif orient == 'vert':
            self.pack(side='top', anchor=NW, padx=padx, pady=pady)     
        self.state = state    
      
    def enter(self, event):
        if self.state != 'disabled':
            self.config(relief='raised', bg=self.bg)

    def leave(self, event):
        if self.state != 'disabled':
            self.config(relief='flat', bg=self.bg)

    def b1down(self, event):
        if self.state != 'disabled':
            self.config(relief='sunken', bg=self.activebackground)
    
    def b1up(self, event):
        if self.state != 'disabled':
            if self.command != None:
                self.command()
            time.sleep(0.05)
            self.config(relief='raised', bg=self.bg)  

    def enable(self):
        self.state = 'normal'

    def disable(self):
        self.state = 'disabled'


class FlatRadiobutton(Label):  
    def __init__(self, parent, group=None, imagefile=None, command=None, value=None, 
                 statushelp=None, balloonhelp=None, orient='horz', buttonMenu=None, 
                 state='normal'):
        Label.__init__(self, parent)
        self.bg = self.cget('bg')
        self.imagefile = imagefile
        self.parent, self.command  = parent, command
        self.group, self.value = group, value
        self.orient = orient
        self.buttonMenu, self.down = buttonMenu, 'false'
        self.Icon = PhotoImage(file=imagefile)
        self.config(image=self.Icon, height=group.bht, width=group.bwd, bd=1)
        if self.value == self.group.variable.get():
            self.config(relief='sunken', bg=group.activebackground)
        self.bind("<Enter>", self.enter)
        self.bind("<Leave>", self.leave)
        self.bind("<ButtonPress-1>", self.b1down)
        self.bind("<ButtonRelease-1>", self.b1up)
        #if group.statusbar != None:
        #    group.statusbar.bind(self.Button, statushelp)
        #if group.balloon != None:
        #    group.balloon.bind(self.Button, balloonhelp)
        if orient == 'horz':
            self.pack(side='left', anchor=NW)
        elif orient == 'vert':
            self.pack(side='top', anchor=NW)
        self.state = state

    def enter(self, event):
        if self.group.variable.get() != self.value and self.state != 'disabled':
            self.config(relief='raised', bg=self.bg)

    def leave(self, event):
        if self.group.variable.get() != self.value and self.state != 'disabled':
            self.config(relief='flat',  bg=self.bg)

    def b1down(self, event):
        if self.group.variable.get() != self.value and self.state != 'disabled':
            self.down = 'true'
            self.command()
            self.group.variable.set(self.value)
            self.config(relief='sunken', bg=self.group.activebackground)
            self.group.poprest(self) 
            #self.update_idletasks()
            #self.after(2000, self.buttonMenu)

    def b1up(self, event):
        if self.group.variable.get() == self.value and self.state != 'disabled':
            self.down = 'false'
            #print 'up'
        
    def buttonMenu(self):
        # after time interval from b1press pop up button menu
        # print 'call buttonMenu'
        if self.down == 'true':
            pass #print 'buttonMenu'

    def enable(self):
        self.state = 'normal'

    def disable(self):
        self.state = 'disabled'
        self.config(relief='flat',  bg=self.bg)
     
  
class FlatRadiogroup:
    def __init__(self, variable, statusbar=None, balloon=None, 
                 bht=21, bwd=21, activebackground='lightgrey'):
        self.buttonList = []
        self.variable = variable
        self.statusbar = statusbar
        self.balloon = balloon
        self.bht = bht
        self.bwd = bwd
        self.activebackground = activebackground

    def addButton(self, parent, imagefile, command, value, statushelp='', balloonhelp='', orient='horz'):
        newButton = FlatRadiobutton(parent, self, imagefile, command, value, statushelp, balloonhelp, orient)
        self.buttonList.append(newButton)
        return newButton     
  
    def poprest(self, pushed):
        for button in self.buttonList:
            if button != pushed:
                button.config(relief='flat', bg='grey')


def printhi():
    print('hi')

if __name__ == '__main__':
    root = Tk()
    root.title('Flatbuttons test')
    from MDI import OrientedFrame
    frame = OrientedFrame(root)
    frame.pack(side=LEFT, anchor=NW)
    open=Flatbutton(frame, imagefile=path+'open.gif', command=printhi)
    v=IntVar()
    v.set(1)
    group = FlatRadiogroup(variable=v)
    open2=group.addButton(frame, imagefile=path+'draw.gif', command=printhi, value=1)
    open3=group.addButton(frame, imagefile=path+'select.gif', command=printhi, value=2)
    root.mainloop()




from tkinter import *
import Pmw
from tkmfly.tkmdi.Widgets.iconPath import path

class Node:
    def __init__(self, parent, tree, menu=None, icon=None, 
                 activeicon=None, name=None, command=None):
        self.parent, self.tree = parent, tree
        self.menu = menu
        self.icon = PhotoImage(file=icon)
        self.activeicon = PhotoImage(file=activeicon)
        self.width, self.height = 1.5*self.icon.width(), 1.5*self.icon.height()
        self.name = name
        self.var = StringVar()
        self.var.set(name)
        self.text = Entry(tree, textvariable=self.var, bg=tree.bg, bd=0, width=len(name)+2, 
            font=tree.font, fg=tree.textcolor, insertwidth=1, highlightthickness=1, 
            highlightbackground=tree.bg, selectbackground="#044484", selectborderwidth=0,
            selectforeground='white')
        self.command = command
        self.x = self.y = 0  #where you draw expandable node, or icon if root or leaf
        self.child = []
        self.state = 'contracted'
        self.selected = 'false'

    def addChild(self, tree, menu=None, icon=None, activeicon=None, name=None, command=None):    
        child = Node(self, tree, menu, icon, activeicon, name, command) 
        self.child.append(child)
        self.tree.display()
        return child         

    def deleteChild(self, child):    
        self.child.remove(child)
        self.tree.display()

    def textForget(self):
        self.text.place_forget()
        for child in self.child:
            child.textForget()                

    def deselect(self):
        self.selected = 'false'
        for child in self.child:
            child.deselect()

    def showmenu(self, event=None):
        x = int(self.tree.winfo_rootx()+self.x)
        y = int(self.tree.winfo_rooty()+self.y)
        self.menu.tk_popup(x, y)

    def boxpress(self, event=None):
        if self.state == 'expanded':
            self.state = 'contracted'
        elif self.state == 'contracted':
            self.state = 'expanded'
        self.tree.display()
 
    def invoke(self, event=None):
        if self.selected == 'false':
            self.tree.deselectall()
            self.selected = 'true'
            self.tree.display()
            self.command()
        self.name = self.text.get()
        self.text.config(width=len(self.name)+2)

    def displayIconText(self):
        tree, text = self.tree, self.text
        if self.selected == 'true':
            self.pic = tree.create_image(self.x, self.y, image=self.activeicon)
        else:
            self.pic = tree.create_image(self.x, self.y, image=self.icon)
        text.place(x=self.x+self.width/2, y=self.y, anchor=W)
        text.bind("<ButtonPress-1>", self.invoke)
        tree.tag_bind(self.pic, "<ButtonPress-1>", self.invoke, "+")
        text.bind("<ButtonPress-3>", self.showmenu)
        tree.tag_bind(self.pic, "<ButtonPress-3>", self.showmenu, "+")
        text.bind("<Double-Button-1>", self.boxpress)
        tree.tag_bind(self.pic, "<Double-Button-1>", self.boxpress, "+")

    def displayRoot(self):
        if self.state == 'expanded':                        
            for child in self.child:
                child.display()            
        self.displayIconText()

    def displayLeaf(self):
        self.tree.hline(self.y, self.parent.x+1, self.x)
        self.tree.vline(self.parent.x, self.parent.y, self.y)
        self.displayIconText()

    def displayBranch(self):
        parent, tree = self.parent, self.tree
        x, y = self.x, self.y            
        tree.hline(y, parent.x, x)           
        tree.vline(parent.x, parent.y, y)
        if self.state == 'expanded' and self.child != []:           
            for child in self.child:
                child.display()                
            box = tree.create_image(parent.x, y, image=tree.minusnode)    
        elif self.state == 'contracted' and self.child != []: 
            box = tree.create_image(parent.x, y, image=tree.plusnode) 
        tree.tag_bind(box, "<ButtonPress-1>", self.boxpress, "+")
        self.displayIconText()

    def findLowestChild(self, node):
        if node.state == 'expanded' and node.child != []:
            return self.findLowestChild(node.child[-1])
        else:
            return node        

    def display(self):
        parent, tree = self.parent, self.tree
        n = parent.child.index(self)
        self.x = parent.x + self.width
        if n == 0:
            self.y = parent.y + (n+1)*self.height
        else:    
            previous = parent.child[n-1]            
            self.y = self.findLowestChild(previous).y + self.height
        if parent == tree:
            self.displayRoot()
        elif parent.state == 'expanded':
            if self.child == []:
                self.displayLeaf() 
            else:        
                self.displayBranch()
            tree.lower('line')


class Tree(Canvas):
    def __init__(self, parent, icon, activeicon, name, menu, command, bg='white', relief='sunken', bd=2, 
                 linecolor='#808080', textcolor='black', font=('MS Sans Serif', 8)):
        Canvas.__init__(self, parent, bg=bg, relief=relief, bd=bd, highlightthickness=0)
        self.pack(side='left', anchor=NW, fill='both', expand=1)
        self.bg, self.font= bg, font
        self.linecolor, self.textcolor= linecolor, textcolor
        self.parent = parent 
        self.plusnode = PhotoImage(file=path+'plusnode.gif')
        self.minusnode = PhotoImage(file=path+'minusnode.gif')
        self.child = []
        self.x = self.y = -10
        self.child.append( Node( self, self, menu=menu, command=command,
            icon=icon, activeicon=activeicon, name=name) )      

    def display(self):
        self.delete(ALL)
        for child in self.child:
            child.textForget()
            child.display()

    def deselectall(self):
        for child in self.child:
            child.deselect()

    def vline(self, x, y, y1):
        for i in range(0, int(abs(y-y1)), 2):
            self.create_line(x, y+i, x, y+i+1, fill=self.linecolor, tags='line')
         
    def hline(self, y, x, x1):
        for i in range(0, int(abs(x-x1)), 2):
            self.create_line(x+i, y, x+i+1, y, fill=self.linecolor, tags='line')


class ScrolledTree(Pmw.ScrolledCanvas):
    def __init__(self, parent, icon, activeicon, name, menu, command, bg='white', 
                 relief='sunken', bd=2, linecolor='#808080', textcolor='black', 
                 font=('MS Sans Serif', 8)):
        Pmw.ScrolledCanvas.__init__(self, parent)
        self.component['canvas'] = Tree(parent, icon=icon, activeicon=activeicon, name=name, menu=menu, command=command)


def printhi():
    print('hi')

if __name__ == "__main__":
    root=Tk()

    m = Pmw.MenuBar(hull_borderwidth = 0, hotkeys=1)
    m.addmenu('File', '')
    m.addmenuitem('File', 'command', label = 'New')
    m.addmenuitem('File', 'command', label = 'Open')
    File = m.component('File-menu')

    f = path+'folder.gif'
    of = path+'openfolder.gif'
    tree = Tree(root, icon=f, activeicon=of, name='tree', menu=File, command=printhi)
    top=tree.child[0]
    top.state='expanded'
    t1=top.addChild(tree, menu=File, icon=f, activeicon=of, name='t1', command=printhi)
    t2=top.addChild(tree, menu=File, icon=f, activeicon=of, name='t2', command=printhi)
    t3=top.addChild(tree, menu=File, icon=f, activeicon=of, name='t3', command=printhi)
    c1=t1.addChild(tree, menu=File, icon=f, activeicon=of, name='c1', command=printhi)
    c2=t2.addChild(tree, menu=File, icon=f, activeicon=of, name='c2', command=printhi)
    c3=t3.addChild(tree, menu=File, icon=f, activeicon=of, name='c3', command=printhi)
    d1=c1.addChild(tree, menu=File, icon=f, activeicon=of, name='d1', command=printhi)  
    d3=c3.addChild(tree, menu=File, icon=f, activeicon=of, name='d3', command=printhi) 

    root.mainloop()


from tkmfly.core import fly_load4, fly_root, fly_local, fly_chdir
from tkinter import Widget, PhotoImage, ttk


def load_datefield():
    from tkmfly.datefield.pkgIndex import code
    fly_root().eval(code)


class DateField(ttk.Entry):
    def __init__(self, master=None, **kw):
        """Constructs a Ttk Entry widget with the parent master.

        STANDARD OPTIONS

            class, cursor, style, takefocus, xscrollcommand

        WIDGET-SPECIFIC OPTIONS

            exportselection, invalidcommand, justify, show, state,
            textvariable, validate, validatecommand, width, format

        VALIDATION MODES

            none, key, focus, focusin, focusout, all
        """
        load_datefield()
        super().__init__(master, "datefield::datefield", **kw)

    def demo(self):
        self.tk.eval("""
::datefield::datefield .df -background yellow -textvariable myDate \
-format "%Y-%m-%d"
pack .df
""")


if __name__ == '__main__':
    from tkinter import Tk, Entry, ttk, Variable

    root = Tk()

    datevar = Variable()

    datefield = DateField(textvariable=datevar, format="%Y-%m-%d")
    print(datefield.get())
    datefield.pack()

    root.mainloop()


from tkinter import *
import Pmw, sys
from tkmfly.tkmdi.Widgets.ProgressBar import ProgressBar
from tkmfly.tkmdi.Widgets.iconPath import path


class MDIChild(Canvas):
    def __init__(self, parent, title='', iconfile=path+'tk.gif', topcolor=None,
                 flickerbg=None, inactivetopcolor=None):
        Canvas.__init__(self, parent.interior, relief='raised', bd=2, 
            width=300, height=200, highlightthickness=0, cursor='arrow')
        if flickerbg == None:
            self.flickerbg = self.cget('bg')
        if sys.platform == 'win32': 
            if topcolor == None:         topcolor='SystemActiveCaption'
            if inactivetopcolor == None: inactivetopcolor='SystemInactiveCaption'
        if topcolor == None:             topcolor="#00008b"           #like win98
        if inactivetopcolor == None:     inactivetopcolor="#808080"   #like win98
        self.topcolor, self.inactivetopcolor = topcolor, inactivetopcolor
        self.id = parent.interior.create_window(0, 0, window=self, anchor="nw")
        self.bind("<Motion>", self.startResize)
        self.bind("<ButtonPress-1>", self.setFocus)
        self.bind("<B1-Motion>", self.resize)
        self.bind("<ButtonRelease-1>", self.stopResize)
        self.parent = parent
        self.state = "normal"
        self.b1motionState = "translating"
        self.lastx = self.lasty = 0

        self.top = self.create_rectangle(3, 3, 700, 20, outline=self.topcolor, fill=self.topcolor)  
        self.tag_bind(self.top, "<ButtonPress-1>", self.press)
        self.tag_bind(self.top, "<Double-Button-1>", self.maximize)
        self.tag_bind(self.top, "<B1-Motion>", self.move)
        self.tag_bind(self.top, "<ButtonPress-3>", self.showmenu)
        self.tag_bind(self.top, "<Motion>", self.arrow)
        self.image = PhotoImage(file=iconfile)
        self.icon = self.create_image(5, 7, image=self.image, anchor=NW)
        self.tag_bind(self.icon, "<ButtonPress-1>", self.showmenu, "+")
        self.tag_bind(self.icon, "<ButtonPress-3>", self.showmenu, "+")
        self.tag_bind(self.icon, "<Motion>", self.arrow, "+")
        if sys.platform == 'win32': ypos=5; font=('MS Sans Serif', 8, 'bold')
        else:                       ypos=7; font=('Helvetica', 9, 'bold')
        self.title = self.create_text(23,ypos, text=title, fill='white', 
            anchor=NW, font=font)
        self.tag_bind(self.title, "<ButtonPress-1>", self.press, "+")
        self.tag_bind(self.title, "<Double-Button-1>", self.maximize)
        self.tag_bind(self.title, "<B1-Motion>", self.move, "+")
        self.tag_bind(self.title, "<ButtonPress-3>", self.showmenu, "+")
        self.tag_bind(self.title, "<Motion>", self.arrow, "+")

        self.closeIcon = PhotoImage(file=path+'close.gif')
        self.closeButton = Button(self, image=self.closeIcon, 
            command=self.close, width=9, height=7)
        self.closeButton.bind("<Motion>", self.arrow)
        self.maximizeIcon = PhotoImage(file=path+'maximize.gif')
        self.maximizeButton = Button(self, image=self.maximizeIcon, 
            command=self.maximize, width=9, height=7)
        self.maximizeButton.bind("<Motion>", self.arrow)
        self.restoreIcon = PhotoImage(file=path+'restore.gif')
        self.restoreButton = Button(self, image=self.restoreIcon, 
            command=self.restore, width=9, height=7)
        self.restoreButton.bind("<Motion>", self.arrow)
        self.minimizeIcon = PhotoImage(file=path+'minimize.gif')
        self.minimizeButton = Button(self, image=self.minimizeIcon, 
            command=self.minimize, width=9, height=7)
        self.minimizeButton.bind("<Motion>", self.arrow)

        w, h = self.winfo_width(), self.winfo_height()
        self.interior = Frame(self, cursor='arrow', bg=self.flickerbg)
        self.interior.place(x=3, y=22, anchor=NW, width=w-7, height=h-27) 
        self.interior.bind("<ButtonPress-1>", self.setFocus)
        self.paint()

        self.parentclose = Button(self.parent.menubar, image=self.closeIcon, 
            command=self.close, width=9, height=7)
        self.parentrestore = Button(self.parent.menubar, image=self.restoreIcon, 
            command=self.restore, width=9, height=7)
        self.parentminimize = Button(self.parent.menubar, image=self.minimizeIcon, 
            command=self.minimize, width=9, height=7)

        m = Pmw.MenuBar(hull_borderwidth = 0, hotkeys=1)
        m.addmenu('State', '')
        m.addmenuitem('State', 'command', label = 'Restore', command=self.restore)
        m.addmenuitem('State', 'command', label = 'Move')
        m.addmenuitem('State', 'command', label = 'Size')
        m.addmenuitem('State', 'command', label = 'Minimize', command=self.minimize)
        m.addmenuitem('State', 'command', label = 'Maximize', command=self.maximize)
        m.addmenuitem('State', 'separator')
        m.addmenuitem('State', 'command', label = 'Close', command=self.close, 
            accelerator="Alt+F4")
        self.menu = m.component('State-menu')
        self.setRestoreState()
        self.setFocus()

    def widgetSetFocus(self, widget):    #Use this after you put pack (etc) children in MDIChild!!
        widget.bind("<ButtonPress-1>", self.setFocus)

    def setFocus(self, event=None):
        print("setFocus")
        p = self.parent
        p.unFocusChildren()
        self.itemconfig(self.top, fill=self.topcolor, 
            outline=self.topcolor)
        p.activechild = self
        Misc.tkraise(self)   

    def showmenu(self, event):
        self.setFocus()
        x = int(self.winfo_rootx()+event.x)
        y = int(self.winfo_rooty()+event.y)
        self.menu.tk_popup(x, y)

    def paint(self, event=None):
        self.update() 
        w, h = self.winfo_width(), self.winfo_height()
        if self.state == 'normal':
            self.restoreButton.place_forget()
            self.closeButton.place(x=w-5, y=5, anchor=NE) 
            self.maximizeButton.place(x=w-23, y=5, anchor=NE) 
            self.minimizeButton.place(x=w-39, y=5, anchor=NE) 
        else:
            self.minimizeButton.place_forget()
            self.closeButton.place(x=w-5, y=5, anchor=NE) 
            self.maximizeButton.place(x=w-23, y=5, anchor=NE) 
            self.restoreButton.place(x=w-39, y=5, anchor=NE) 
        self.interior.place(x=3, y=22, anchor=NW, width=w-7, height=h-27) 

    def arrow(self, event):
        self.config(cursor='arrow')

    def changeCursor(self, win32=None, other=None):
        if sys.platform == 'win32' and win32 != None: 
            self.config(cursor=win32)            
        else: 
            self.config(cursor=other) 
 
    def startResize(self, event):
        ex, ey = event.x, event.y
        w, h = self.winfo_width(), self.winfo_height()
        t = 5
        if (ey < 20 and ex < t) or (ey < t and ex < 20):			
            self.changeCursor(win32='size_nw_se', other='top_left_corner') 
            self.b1motionState = "topleft" 
        elif (ey > h-20 and ex > w-t) or (ey > h-t and ex > w-20):
            self.changeCursor(win32='size_nw_se', other='bottom_right_corner') 
            self.b1motionState = "bottomright" 	
        elif (ey < 20 and ex > w-t) or (ey < t and ex > w-20):		
            self.changeCursor(win32='size_ne_sw', other='top_right_corner')
            self.b1motionState = "topright"  
        elif (ey > h-20 and ex < t) or (ey > h-t and ex < 20):	
            self.changeCursor(win32='size_ne_sw', other='bottom_left_corner')	
            self.b1motionState = "bottomleft"  
        elif (ey < t):
            self.changeCursor(win32='sb_v_double_arrow', other='top_side')
            self.b1motionState = "top" 
        elif (ey > h-t):
            self.changeCursor(win32='sb_v_double_arrow', other='bottom_side')
            self.b1motionState = "bottom" 
        elif (ex < t):
            self.changeCursor(win32='sb_h_double_arrow', other='left_side')
            self.b1motionState = "left" 
        elif (ex > w-t):
            self.changeCursor(win32='sb_h_double_arrow', other='right_side')
            self.b1motionState = "right" 
        else:
            self.b1motionState = "translating" 

    def stopResize(self, event):
        self.b1motionState = "translating"

    def resize(self, event):
        if self.b1motionState == "translating":
            return
        canvas = self.parent.interior
        w, h = canvas.winfo_width(), canvas.winfo_height()
        xmouse, ymouse = self.winfo_x()+event.x, self.winfo_y()+event.y
        if xmouse < 0 or xmouse > w or ymouse < 0 or ymouse > h: 
            return
        if self.b1motionState == "topleft":
            self.setSize(topPos=ymouse, leftPos=xmouse)
        elif self.b1motionState == "bottomleft":
            self.setSize(bottomPos=ymouse, leftPos=xmouse)
        elif self.b1motionState == "topright":
            self.setSize(topPos=ymouse, rightPos=xmouse)
        elif self.b1motionState == "bottomright":
            self.setSize(bottomPos=ymouse, rightPos=xmouse)
        elif self.b1motionState == "top":
            self.setSize(topPos=ymouse)
        elif self.b1motionState == "bottom":
            self.setSize(bottomPos=ymouse)
        elif self.b1motionState == "left":
            self.setSize(leftPos=xmouse)
        elif self.b1motionState == "right":
            self.setSize(rightPos=xmouse)

    def setSize(self, leftPos=None, rightPos=None, topPos=None, bottomPos=None):
        w, h = self.winfo_width()-4, self.winfo_height()-4
        x, y = self.winfo_x(), self.winfo_y()
        if rightPos == None:
            rightPos = x+w
        else:
            rightPos = max(rightPos, x+100) 
        if leftPos == None:
            leftPos = x
        else:
            leftPos = min(leftPos, rightPos-100)
        if bottomPos == None:
            bottomPos = y+h
        else:
            bottomPos = max(bottomPos, y+19)
        if topPos == None:
            topPos = y
        else:
            topPos = min(topPos, bottomPos-19)
        self.parent.interior.coords(self.id, leftPos, topPos)
        self.config(width=rightPos-leftPos, height=bottomPos-topPos)
        self.superUpdate()

    def superUpdate(self):
        self.setRestoreState()
        self.restore()        

    def press(self, event):
        self.lastx, self.lasty = event.x, event.y

    def move(self, event):
        if self.b1motionState == "translating":
            canvas = self.parent.interior
            w, h = canvas.winfo_width(), canvas.winfo_height()
            xmouse, ymouse = self.winfo_x()+event.x, self.winfo_y()+event.y
            if xmouse > 0 and xmouse < w and ymouse > 0 and ymouse < h:
                x = event.x_root - canvas.winfo_rootx() - self.lastx
                y = event.y_root - canvas.winfo_rooty() - self.lasty
                canvas.coords(self.id, x, y)

    def delIcons(self):
        self.parentclose.pack_forget()
        self.parentrestore.pack_forget()
        self.parentminimize.pack_forget()

    def setRestoreState(self):
        if self.state == 'normal':
            self.restoreX, self.restoreY = self.winfo_x(), self.winfo_y()
            self.restoreWidth, self.restoreHeight = self.winfo_width()-4, self.winfo_height()-4
            self.restoreState = self.state

    def minimize(self):
        self.setRestoreState()
        self.delIcons()
        p = self.parent
        p.nonMinimized.remove(self)
        p.minimizedChildren.append(self)
        p.repackMinimized()
        self.config(width=150, height=19)
        self.state = 'minimized'
        if len(p.childlist) > 0:
            p.childlist[0].setFocus()
        self.paint()

    def maximize(self, event=None):
        if self in self.parent.minimizedChildren:
            self.parent.minimizedChildren.remove(self)
        self.setRestoreState()
        self.state = 'maximized'
        canvas = self.parent.interior
        self.config(width=canvas.winfo_width()+7, height=canvas.winfo_height()+26)
        canvas.coords(self.id, -6, -25) 
        self.parentclose.pack(side='right', padx=2)
        self.parentrestore.pack(side='right')
        self.parentminimize.pack(side='right')
        self.parent.activechild = self
        self.paint()

    def restore(self):
        if self in self.parent.minimizedChildren:
            self.parent.minimizedChildren.remove(self)
            self.parent.nonMinimized.append(self)
        if self.restoreState == 'maximized':
            self.maximize()
        else:
            self.state = 'normal'
            self.delIcons()
            canvas = self.parent.interior
            canvas.coords(self.id, self.restoreX, self.restoreY)
            self.config(width=self.restoreWidth, height=self.restoreHeight)
            self.paint()

    def close(self):
        self.parentclose.destroy()
        self.parentrestore.destroy()
        self.parentminimize.destroy()
        p = self.parent
        p.childlist.remove(self)
        p.nonMinimized.remove(self)
        if len(p.childlist) > 0:
            p.childlist[0].setFocus()
        else:
            p.activechild = None
        self.image = self.minimizeIcon = None
        self.maximizeIcon = self.closeIcon = self.restoreIcon = None
        self.destroy()


class OrientedFrame(Frame):
    def __init__(self, parent, orient='horz', **kw):
        Frame.__init__(self, parent, kw)
        self.orient = orient
          

class MDIParent(Tk):
    def __init__(self, title='', interior_bg='#808080'):
        Tk.__init__(self)
        self.interior_bg = interior_bg
        self.title(title)
        self.lastx = self.lasty = None
        self.childlist = []
        self.activechild = None        

        self.menubar = Frame(self)
        self.menubar.pack(fill='x')

        self.topline = Canvas(self, bg='grey', height=2)
        self.topline.pack(fill='x')
        self.topline.create_line(0,2,1000,2, fill='#808080') 
        self.topline.create_line(0,3,1000,3, fill='white')      
        self.topspace = OrientedFrame(self, orient='horz')
        self.topspace.pack(fill='x')
        
        self.middle = Canvas(self, bg='grey', highlightthickness=0)
        self.middle.pack(fill='both', expand=1)
        self.middle.bind("<Configure>", self.paint)
        self.interior = Canvas(self.middle, bg=self.interior_bg, highlightthickness=0)
        self.leftspace = OrientedFrame(self.middle, orient='vert')
        self.leftspace.pack(side='left', fill='y', pady=4)
        self.rightspace = OrientedFrame(self.middle, orient='vert')
        self.rightspace.pack(side='right', fill='y', pady=4)

        self.statusbar = Frame(self)
        self.statusbar.pack(side='bottom', fill='x')
        self.cornerIcon = PhotoImage(file=path+'corner.gif')
        self.corner = Label(self.statusbar, image=self.cornerIcon)
        self.corner.pack(side='right', anchor=SE)        
        self.progress = ProgressBar(self.statusbar, height=12)
        self.progress.pack(side='right', padx=11, anchor=SE)   
        self.geometry("%dx%d" % (400,300))
        self.lastw = self.lasth = None
        self.minimizedChildren = []
        self.nonMinimized = []

    def paint(self, event=None):
        self.middle.delete(ALL)
        h = self.middle.winfo_height()
        w = self.middle.winfo_width()
        lw = self.leftspace.winfo_width()+1
        rw = self.rightspace.winfo_width()+2

        self.middle.create_line(0,2,w,2, fill='#808080') 
        self.middle.create_line(0,3,w,3, fill='white') 
        self.middle.create_line(lw,3,w-rw,3, fill='black') 

        self.middle.create_line(0,h-4,w,h-4, fill='#808080') 
        self.middle.create_line(lw,h-4,w-rw,h-4, fill='lightgrey') 
        self.middle.create_line(0,h-3,w,h-3, fill='white') 

        self.middle.create_line(lw-1,3,lw-1,h-4, fill='#808080') 
        self.middle.create_line(lw,3,lw,h-4, fill='black') 

        self.middle.create_line(w-rw,3,w-rw,h-3, fill='lightgrey') 
        self.middle.create_line(w-rw+1,3,w-rw+1,h-3, fill='white') 
        
        self.middle.create_rectangle(lw+1,4,w-rw-1,h-5, fill=self.interior_bg, 
            outline=self.interior_bg)
        self.interior.config(width=w-rw-lw-1, height=h-8)
        self.interior.place(x=lw+1, y=4, anchor=NW)  

        # make maximized children's w, h follow resizing in x and y
        if self.lastw != None:
            for child in self.childlist:
                if child.state == 'maximized':
                    cw, ch = child.winfo_width(), child.winfo_height()
                    child.config(width=cw+w-self.lastw, height=ch+h-self.lasth)
        self.repackMinimized()
        self.lastw, self.lasth = w, h

    def unFocusChildren(self):   
        for child in self.childlist:
            child.itemconfig(child.top, fill=child.inactivetopcolor, 
                outline=child.inactivetopcolor)
        
    def newChild(self, title='tk', iconfile=path+'tk.gif'):
        state = None
        if self.activechild != None:
            state = self.activechild.state
            if state == 'maximized':
                self.activechild.restore()
        self.activechild = MDIChild(self, title=title, iconfile=iconfile)
        if state == 'maximized': 
            self.activechild.maximize()
        self.childlist.append(self.activechild)
        self.nonMinimized.append(self.activechild)
        return self.activechild

    def repackMinimized(self):
        for window in self.minimizedChildren:
            index = self.minimizedChildren.index(window)
            w, h = self.interior.winfo_width(), self.interior.winfo_height()
            xslots = int(w/154)
            if xslots == 0: xslots = 1
            layer = int(index/xslots)     # from 0 and up
            xslot = index%xslots  		# from 0 to xslots-1
            ycoord = h-(layer+1)*23
            self.interior.coords(window.id, xslot*154, ycoord)
        if self.minimizedChildren == []:
            return self.interior.winfo_height()
        else:
            return ycoord

    def cascadeChildren(self):
        self.tileChildren()
        i = 0
        for child in self.nonMinimized:
            if child != self.activechild:
                self.interior.coords(child.id, i*21, i*21)                
                child.config(width=300, height=200)
                Misc.tkraise(child)
                child.superUpdate()
                i = i + 1
        self.interior.coords(self.activechild.id, i*21, i*21)
        self.activechild.config(width=300, height=200)
        Misc.tkraise(self.activechild)
        self.activechild.superUpdate()

    def tileChildrenSystematically(self):         
        tot = len(self.nonMinimized)
        slots = int(tot/2)  
        w = self.interior.winfo_width()/slots
        h = self.repackMinimized()/slots
        for window in self.nonMinimized:
            index = self.nonMinimized.index(window)
            layer = int(index/slots)         # from 0 and up
            slot = index%slots  		   # from 0 to slots-1           
            self.interior.coords(window.id, w*slot, h*layer)              
            window.config(width=w-4, height=h-4) 
            window.superUpdate()

    def tileChildren(self):
        w = self.interior.winfo_width()
        h = self.repackMinimized()        
        tot = len(self.nonMinimized)
        if tot == 1:
            for child in self.nonMinimized:
                self.interior.coords(child.id, 0, 0)                
                child.config(width=w-4, height=h-4)
        elif tot == 2:
            i = 0
            for child in self.nonMinimized:
                self.interior.coords(child.id, i*w/2, 0)                
                child.config(width=w/2-4, height=h-4)
                i = i + 1
        elif tot == 3:
            i = 0
            for child in self.nonMinimized:
                if i == 0:
                    self.interior.coords(child.id, 0, 0)                
                    child.config(width=w/2-4, height=h-4)
                elif i == 1 or i == 2:
                    self.interior.coords(child.id, w/2, (i-1)*h/2)                
                    child.config(width=w/2-4, height=h/2-4)
                i = i + 1
        elif tot == 4:
            i = 0
            for child in self.nonMinimized:
                if i == 0 or i == 1:
                    self.interior.coords(child.id, 0, i*h/2)                
                elif i == 2 or i == 3:
                    self.interior.coords(child.id, w/2, (i-2)*h/2)               
                child.config(width=w/2-4, height=h/2-4)
                i = i + 1
        for child in self.nonMinimized:
            child.superUpdate()
        if tot > 4:
            self.tileChildrenSystematically()


def printhi():
    print('hi')


def putTree(document):
    m = Pmw.MenuBar(hull_borderwidth = 0, hotkeys=1)
    m.addmenu('File', '')
    m.addmenuitem('File', 'command', label = 'New')
    m.addmenuitem('File', 'command', label = 'Open')
    File = m.component('File-menu')

    f = path+'folder.gif'
    of = path+'openfolder.gif'
    tree = Tree(document.interior, icon=f, activeicon=of, name='tree', menu=File, command=printhi)
    top=tree.child[0]
    top.state='expanded'
    t1=top.addChild(tree, menu=File, icon=f, activeicon=of, name='t1', command=printhi)
    t2=top.addChild(tree, menu=File, icon=f, activeicon=of, name='t2', command=printhi)
    t3=top.addChild(tree, menu=File, icon=f, activeicon=of, name='t3', command=printhi)
    c1=t1.addChild(tree, menu=File, icon=f, activeicon=of, name='c1', command=printhi)
    c2=t2.addChild(tree, menu=File, icon=f, activeicon=of, name='c2', command=printhi)
    c3=t3.addChild(tree, menu=File, icon=f, activeicon=of, name='c3', command=printhi)
    d1=c1.addChild(tree, menu=File, icon=f, activeicon=of, name='d1', command=printhi)  
    d3=c3.addChild(tree, menu=File, icon=f, activeicon=of, name='d3', command=printhi) 
    
    document.widgetSetFocus(tree)  ## important for setting focus to MDIChild ##



if __name__ == '__main__':
    from FlatButtons import FlatRadiogroup
    from Tree import Tree
    from Toolbar import Toolbar

    App=MDIParent(title='MDIParent')
    App.geometry("%dx%d" % (500,400))
    spaces = [App.topspace, App.leftspace, App.rightspace]

    v = StringVar()
    v.set("line")
    t = Toolbar(App.topspace, dockingspaces=spaces, title="Sketch")

    stateGroup = FlatRadiogroup(variable=v)
    t.sendFlatRadioGroup(groupname='stateGroup', group=stateGroup)
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'select.gif', valuename='select')
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'draw.gif', valuename='line')
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'circle.gif', valuename='circle')
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'rect.gif', valuename='rect')
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'cut.gif', valuename='cut')
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'zoomToArea.gif', valuename='toarea')
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'zoom.gif', valuename='zoom')  
    t.addFlatRadiobutton(groupname='stateGroup', imagefile=path+'trans.gif', valuename='translate')
    
    menuBar = Pmw.MenuBar(App.menubar, hull_borderwidth=0, hotkeys=1)
    menuBar.pack(side='left', fill='x')
    File = menuBar.addmenu('File', 'Hi')
    menuBar.addmenuitem('File', 'command', label='New', command=App.newChild)
    menuBar.addmenuitem('File', 'separator')
    menuBar.addmenuitem('File', 'command', label='Cascade', command=App.cascadeChildren)
    menuBar.addmenuitem('File', 'command', label='Tile', command=App.tileChildren)

    teststatusbar = Label(App.statusbar, text='Statusbar')
    teststatusbar.pack(side='left')    
    
    child1 = App.newChild(title='MDIChild1', iconfile=path+'tk.gif')
    putTree(child1)      
    child2 = App.newChild(title='MDIChild2', iconfile=path+'tk.gif') 
    putTree(child2) 

    App.mainloop()




from tkmfly.core import fly_load4, fly_root, fly_local, fly_chdir
from tkinter import Widget, PhotoImage


def load_notifywindow():
    from tkmfly.notifywindow.pkgIndex import code
    fly_root().eval(code)


def notifywindow_notifywindow(message: str = "", image=None):
    load_notifywindow()
    fly_root().call("::notifywindow::notifywindow", message, image)


def notifywindow_demo():
    load_notifywindow()
    fly_root().call("::notifywindow::demo")


class NotifyWindow(object):
    def demo(self):
        notifywindow_demo()

    def popup(self, message: str, image=None):
        if image is None:
            image = "info"
        notifywindow_notifywindow(message, image)


if __name__ == '__main__':
    from tkinter import Tk, Entry, ttk

    root = Tk()

    notifywindow = NotifyWindow()
    notifywindow.popup("hello")

    root.mainloop()



code = """
# history.tcl --
#
#       Provides a history mechanism for entry widgets
#
# Copyright (c) 2005    Aaron Faupell <afaupell@users.sourceforge.net>
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
# 
# RCS: @(#) $Id: history.tcl,v 1.4 2005/08/25 03:36:58 andreas_kupries Exp $

package require Tk
package provide history 0.1

namespace eval history {
    bind History <Up>   {::history::up %W}
    bind History <Down> {::history::down %W}
}

proc ::history::init {w {len 30}} {
    variable history
    variable prefs
    set bt [bindtags $w]
    if {[lsearch $bt History] > -1} { error "$w already has a history" }
    if {[set i [lsearch $bt $w]] < 0} { error "cant find $w in bindtags" }
    bindtags $w [linsert $bt [expr {$i + 1}] History]
    array set history [list $w,list {} $w,cur -1]
    set prefs(maxlen,$w) $len
    return $w
}

proc ::history::remove {w} {
    variable history
    variable prefs
    set bt [bindtags $w]
    if {[set i [lsearch $bt History]] < 0} { error "$w has no history" }
    bindtags $w [lreplace $bt $i $i]
    unset prefs(maxlen,$w) history($w,list) history($w,cur)
}

proc ::history::add {w line} {
    variable history
    variable prefs
    if {$history($w,cur) > -1 && [lindex $history($w,list) $history($w,cur)] == $line} {
        set history($w,list) [lreplace $history($w,list) $history($w,cur) $history($w,cur)]
    }
    set history($w,list) [linsert $history($w,list) 0 $line]
    set history($w,list) [lrange $history($w,list) 0 $prefs(maxlen,$w)]
    set history($w,cur) -1
}

proc ::history::up {w} {
    variable history
    if {[lindex $history($w,list) [expr {$history($w,cur) + 1}]] != ""} {
        if {$history($w,cur) == -1} {
            set history($w,tmp) [$w get]
        }
        $w delete 0 end
        incr history($w,cur)
        $w insert end [lindex $history($w,list) $history($w,cur)]
    } else {
        alert $w
    }
}

proc ::history::down {w} {
    variable history
    if {$history($w,cur) != -1} {
        $w delete 0 end
        if {$history($w,cur) == 0} {
            $w insert end $history($w,tmp)
            set history($w,cur) -1
        } else {
            incr history($w,cur) -1
            $w insert end [lindex $history($w,list) $history($w,cur)]
        }
    } else {
        alert $w
    }
}

proc ::history::get {w} {
    variable history
    return $history($w,list)
}

proc ::history::clear {w} {
    variable history
    set history($w,cur) -1
    set history($w,list) {}
    unset -nocomplain history($w,tmp)
}

proc ::history::configure {w option {value {}}} {
    variable history
    variable prefs
    switch -exact -- $option {
        length {
            if {$value == ""} { return $prefs(maxlen,$w) }
            if {![string is integer -strict $value]} { error "length must be an integer" }
            set prefs(maxlen,$w) $value
        }
        alert {
            if {$value == ""} { return [info body ::history::alert] }
            proc ::history::alert w $value
        }
        default {
            error "unknown option $option"
        }
    }
}

proc ::history::alert {w} {bell}

"""


from tkinter import *
from tkmfly.tkmdi.Widgets.FlatButtons import Flatbutton, FlatRadiogroup
from tkmfly.tkmdi.Widgets.iconPath import path

class OrientedToplevel(Toplevel):
    def __init__(self, orient='horz'):
        Toplevel.__init__(self)
        self.orient = orient

class Toolbar:
    def __init__(self, parent, dockingspaces=None, title=None):
        self.parent = parent
        self.dockingspaces = dockingspaces
        self.title = title
        self.buttons = []
        self.docked = 'true'
        self.frame = Frame() 
        self.float = Toplevel()
        self.float.withdraw()
        self.horztab = PhotoImage(file=path+'horztab.gif')
        self.activehorztab = PhotoImage(file=path+'activehorztab.gif')
        self.verttab = PhotoImage(file=path+'verttab.gif')
        self.activeverttab = PhotoImage(file=path+'activeverttab.gif')
        self.horzsep = PhotoImage(file=path+'horzsep.gif')
        self.vertsep = PhotoImage(file=path+'vertsep.gif')
        self.none = lambda: None

    def dock(self, space):
        self.docked = 'true'
        self.parent = space
        self.attach(space)
        self.float.destroy()

    def undock(self, event):
        self.docked = 'false'
        self.float = OrientedToplevel()
        self.float.orient = self.parent.orient
        self.float.transient(self.parent)
        self.float.resizable(0,0)
        self.attach(self.float)
        self.float.focus_set()
        x, y = self.parent.winfo_rootx(), self.parent.winfo_rooty()
        self.float.geometry("+%d+%d" % (x+event.x, y+event.y))

    def b1down(self, event):
        if self.docked == 'true':
            self.undock(event)

    def b1motion(self, event):
        if self.docked == 'false':
            x, y = self.float.winfo_x(), self.float.winfo_y()
            if x != 0:
                self.float.geometry("+%d+%d" % (x+event.x, y+event.y))
         
    def b1up(self, event):
        if self.docked == 'false':
            for space in self.dockingspaces:
                if self.insideSpace(space) == 'true':
                    self.dock(space)
                    break

    def insideSpace(self, space):
        spacex0, spacey0 = space.winfo_rootx(), space.winfo_rooty()
        spacex1, spacey1 = spacex0 + space.winfo_width(), spacey0 + space.winfo_height()        
        x, y = self.float.winfo_rootx(), self.float.winfo_rooty()
        if x > spacex0 and x < spacex1 and y > spacey0 and y < spacey1:
            return 'true'
        else:
            return 'false'   

    def entertab(self, event):
        if self.parent.orient == 'horz':
            self.tab.config(image=self.activehorztab)
        else:
            self.tab.config(image=self.activeverttab)

    def leavetab(self, event):
        if self.parent.orient == 'horz':
            self.tab.config(image=self.horztab)
        else:
            self.tab.config(image=self.verttab)
             
    def attach(self, space):
        self.frame.destroy()
        self.frame = Frame(space)
        if space.orient == 'horz':
            self.float.title(self.title) 
            self.frame.pack(side='left', anchor=NW)
            self.tab = Label(self.frame, image=self.horztab)
            self.tab.pack(side='left', anchor=NW)
        else:
            self.float.title('')
            self.frame.pack(side='top', anchor=NW)
            self.tab = Label(self.frame, image=self.verttab)
            self.tab.pack(side='top', anchor=NW)
        self.tab.bind("<Enter>", self.entertab)
        self.tab.bind("<Leave>", self.leavetab)
        self.tab.bind("<ButtonPress-1>", self.b1down)
        self.tab.bind("<B1-Motion>", self.b1motion)
        self.tab.bind("<ButtonRelease-1>", self.b1up)
        for line in self.buttons: 
            #print line
            exec(line)

    def addFlatbutton(self, imagefile=None, commandname=None):
        line = "Flatbutton(self.frame, imagefile='%s', command=self.%s, \
            orient=self.parent.orient)" % (imagefile, commandname)
        self.buttons.append(line)
        self.attach(self.parent)

    def addFlatRadiobutton(self, groupname=None, imagefile=None, commandname='none', valuename=None):
        # before you create a new Modebutton, 
        # delete reference to old one in its group.buttonList
        # use imagefile as identifier
        line =        "for button in self.%s.buttonList[:]:      \n" % groupname
        line = line + "    if button.imagefile == '%s':          \n" % imagefile 
        line = line + "        self.%s.buttonList.remove(button) \n" % groupname 
        self.buttons.append(line)         
        self.buttons.append( "self.%s.addButton(self.frame, imagefile='%s', command=self.%s, value='%s', \
            orient=self.parent.orient)" % (groupname, imagefile, commandname, valuename) )
        self.attach(self.parent)

    def addSeparator(self):
        self.buttons.append( "if space.orient == 'horz': \
            Label(self.frame, image=self.horzsep).pack(side='left', anchor=NW)" )
        self.buttons.append( "if space.orient == 'vert': \
            Label(self.frame, image=self.vertsep).pack(side='top', anchor=NW)" )
        self.attach(self.parent)

    def sendFlatRadioGroup(self, groupname=None, group=None):
        exec("self.%s = group" % groupname)

    def sendCommand(self, commandname=None, command=None):
        exec("self.%s = command" % commandname)
        
def printhi():
    print('hi')

if __name__ == '__main__':
    from MDI import MDIParent

    App = MDIParent(title='MDI')
    spaces = [App.topspace, App.leftspace, App.rightspace]

    t1 = Toolbar(App.topspace, dockingspaces=spaces, title='File')
    t1.sendCommand(commandname='printhi', command=printhi)
    t1.addFlatbutton(imagefile=path+'new.gif', commandname='printhi')
    t1.addFlatbutton(imagefile=path+'open.gif', commandname='printhi')
    t1.addFlatbutton(imagefile=path+'save.gif', commandname='printhi')

    v=IntVar()
    v.set(1)
    t2 = Toolbar(App.leftspace, dockingspaces=spaces, title='Edit')
    viewGroup = FlatRadiogroup(variable=v)
    t2.sendFlatRadioGroup(groupname='viewGroup', group=viewGroup)
    t2.sendCommand(commandname='printhi', command=printhi)
    t2.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'draw.gif', commandname='printhi', valuename='1')
    t2.addSeparator() 
    t2.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'select.gif', commandname='printhi', valuename='2')
    t2.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'rect.gif', commandname='printhi', valuename='3')
    t2.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'circle.gif', commandname='printhi', valuename='4')

    d=IntVar()
    d.set(1)
    t3 = Toolbar(App.rightspace, dockingspaces=spaces, title='Display')
    displayGroup = FlatRadiogroup(variable=d)
    t3.sendFlatRadioGroup(groupname='displayGroup', group=displayGroup)
    t3.sendCommand(commandname='printhi', command=printhi)
    t3.addFlatRadiobutton(groupname='displayGroup', imagefile=path+'wire.gif', commandname='printhi', valuename='1')
    t3.addFlatRadiobutton(groupname='displayGroup', imagefile=path+'stick.gif', commandname='printhi', valuename='2')
    t3.addFlatRadiobutton(groupname='displayGroup', imagefile=path+'ballstick.gif', commandname='printhi', valuename='3')
    t3.addFlatRadiobutton(groupname='displayGroup', imagefile=path+'sphere.gif', commandname='printhi', valuename='4')
    t3.addFlatRadiobutton(groupname='displayGroup', imagefile=path+'surface.gif', commandname='printhi', valuename='5')

    t4 = Toolbar(App.topspace, dockingspaces=spaces, title='View')
    t4.sendCommand(commandname='printhi', command=printhi)
    t4.sendFlatRadioGroup(groupname='viewGroup', group=viewGroup)
    t4.addFlatbutton(imagefile=path+'toscreen.gif', commandname='printhi')
    t4.addFlatbutton(imagefile=path+'toselection.gif', commandname='printhi')
    t4.addSeparator()
    t4.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'zoomToArea.gif', commandname='printhi', valuename='5')
    t4.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'zoom.gif', commandname='printhi', valuename='6')
    t4.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'rotate.gif', commandname='printhi', valuename='7')
    t4.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'rotz.gif', commandname='printhi', valuename='8')
    t4.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'trans.gif', commandname='printhi', valuename='9')
    t4.addFlatRadiobutton(groupname='viewGroup', imagefile=path+'zclip.gif', commandname='printhi', valuename='10')  

    App.mainloop()